"use strict";

var invariant = require("invariant");

var writtenNumber = require("written-number/lib/standalone");

function writtenNumberCurrency() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref = config || {},
      lang = _ref.lang,
      amountArg = _ref.amount,
      precisionArg = _ref.precision,
      amountFormatter = _ref.amountFormatter,
      _ref$linker = _ref.linker,
      linker = _ref$linker === void 0 ? "" : _ref$linker,
      precisionFormatter = _ref.precisionFormatter;

  invariant(lang, "config.lang is required");
  writtenNumber.defaults.lang = lang;
  return function () {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        amountArg = _ref2.amount,
        precisionArg = _ref2.precision;

    var amount, precision;

    if (amountArg == null && precisionArg != null) {
      precision = prepareNumber(precisionArg, "precision");
      validateFunction(precisionFormatter, "precisionFormatter");
    } else if (amountArg != null && precisionArg == null) {
      amount = prepareNumber(amountArg, "amount");
      validateFunction(amountFormatter, "amountFormatter");
    } else if (amountArg == null && precisionArg == null) {
      invariant(false, "Please provide a amount or a precision");
    } else {
      amount = prepareNumber(amountArg, "amount");
      precision = prepareNumber(precisionArg, "precision");
      validateFunction(amountFormatter, "amountFormatter");
      validateFunction(precisionFormatter, "precisionFormatter");
    }

    if (amount == null) {
      return format(precision, precisionFormatter);
    }

    if (precision == null) {
      return format(amount, amountFormatter);
    }

    return link(format(amount, amountFormatter), format(precision, precisionFormatter), linker);
  };
}

function link(left, right, linker) {
  return linker ? "".concat(left, " ").concat(linker, " ").concat(right) : "".concat(left, " ").concat(right);
}

function format(amount, formatter) {
  return writtenNumber(amount) + " " + formatter(amount);
}

function validateFunction(n, name) {
  if (typeof n !== "function") {
    invariant(false, "config.".concat(name, " should be a valid function"));
  }
}

function prepareNumber(n, name) {
  n = Number(n);

  if (Number.isNaN(n)) {
    invariant(false, "config.".concat(name, " should be valid number"));
  }

  if (typeof n === "number" && n < 0) {
    invariant(false, "config.".concat(name, " should be greater than or equal to zero"));
  }

  return Math.round(n);
}

module.exports = writtenNumberCurrency;